** Program   : DemoPick.prg
** Written   : 12/13/89
** Modified  : 03/07/90
** Author    : Richard H. Price
** Purpose   : Demonstrate multiple field popup pick lists which follow
**             the basic conventions of dBase IV's POPUP..PROMPT..FIELD
**             routine. Allow user to lookup data in a dbf in the form of a
**             look list, fill an active GET <memvar_or_field> from the lookup
**             table choice.
**
**             This demo program also shows how to use READ with a FORMAT
**             file to mimic EDIT via PROCEDURE REC_READ.
**
**             Data validation is performed throughout via a single
**             UDF (Function PickFunc). The universal nature of the UDF
**             allows both pick lists and data validation through the use
**             of procedures found within POPPICK4. Programmers need only
**             maintain PROCEDURE PickCase for "universal" pick lists and data
**             validation purposes.
**
** Modified  : January 26, 1990
**             Modifications make use of a binary module called KEYBD.bin
**             which provides a method of stuffing the keyboard. All references
**             to PLAY MACRO found in prior version (POPPICK3) have been
**             deleted and replaced with CALL KEYBD WITH <expC>.
**             Also, a bug fix has been implemented to self correct for listings
**             in a window where the window may accomodate more choices than
**             are to be found within the chosen dbf. More specifically, when
**             a list begins in a window, it always starts at position 1,0
**             thereby necessitating the program to shift the beginning data
**             up to position 0,0.  This is accomplished by placing the cursor
**             at the bottom row of the window and issueing a ? (print).
**             It was found that when the length of the LIST object equaled
**             the width of the window, a subsequent ? (print) was ignored.
**             The workaround is IF COL()=0, issue a ? to clear the bug so that
**             the positioning ? will shift correctly.
**
** Format    : POPNPICK.fmo
**
** DBF Files : POPORDER.dbf and .mdx (TAGS: Lname1 , Lname2)
**           : PICKPART.dbf and .mdx (TAG: ID)     ** id
**           : PICKNAME.dbf and .mdx (TAG: L_Name) ** l_name
**
** Binary    : CURSOR.bin
**           : KEYBD.bin
**
** Procedures: Held in this file (PickDemo.prg):
**             Used to demonstrate the pick lists are:
**
**                  PROCEDURE DemoPick  -  Main demonstration program
**                  PROCEDURE Set_Env   -  Set the operating environment
**                  PROCEDURE D_WinPop  -  Define windows & popups
**                  PROCEDURE Reset_Env -  Reset the operating environment
**                  PROCEDURE Pop_Fmt   -  A screen formatting procedure.
**
** SEE ALSO POPPICK5.prg

PROCEDURE DemoPick
   set trap on
   CLOSE DATABASES
   ** Initialize environmental variables
   STORE .F. TO Old_con,Old_near, Old_stat, Old_bell, Old_score, Old_talk,;
   Old_esc, Old_delet, Old_exact, Old_delim, Old_head

   *-- Set Operating Environment
   DO Set_env

   *-- Define The WINDOWS Used FOR Pick Lists and define POPUPS
   *-- for demo Of POPUP..Prompt..FIELD Pick List
   DO D_WinPop

   *-- Loop thru until user Quits via "Q" or Esc
   DO WHILE .t.
      SET CONFIRM OFF
      clear type
      i=inkey()
      M_Typeread="F"
      @ 7,20 TO 11,60
      @ 9,21 TO 9,59
      @ 8,22 say  "  Use a FORMAT File or Procedure "
      @ 10,22 say " Please enter your choice (F/P): " ;
         get M_Typeread pict "!" VALID M_TypeRead$"FP" ;
         message " Esc to quit.... " ;
         ERROR " Enter F-Format, P-Procedure, (Esc-to quit) "
      read
      SELECT Poporder
      DO CASE
         CASE READKEY()=12
            EXIT
         CASE M_Typeread="F"
            *-- This example uses a format file.

            *-- This procedure uses READ and mimics EDIT by
            *-- processing the user's navigational keystrokes.

            *-- Procedure PickEdit is found in the procedure file POPPICK5.prg
            *-- Its purpose is to mimic the command: EDIT NOMENU
            DO PickEdit WITH "POPNPICK",.t.

         CASE M_Typeread="P"
            *-- This example uses a format type of procedure.

            *-- This procedure uses READ and mimics EDIT by
            *-- processing the user's navigational keystrokes.

            *-- Procedure PickEdit is found in the procedure file POPPICK5.prg
            *-- Its purpose is to mimic the command: EDIT NOMENU
            DO PickEdit WITH "POP_FMT",.f.
      ENDCASE
      *-- Deactivate any active format file.
      SET FORMAT TO
   ENDDO

   *-- Reset environment to settings found when entering program
   DO Reset_env
RETURN

PROCEDURE Set_env
   set color to w+/b,rg+/r
   Old_con   = IIF(SET("CONFIRM")="ON",.T.,.F.)
   Old_near  = IIF(SET("NEAR")="ON",.T.,.F.)
   Old_stat  = IIF(SET("STATUS")="ON",.T.,.F.)
   Old_bell  = IIF(SET("BELL")="ON",.T.,.F.)
   Old_score = IIF(SET("SCOREBOARD")="ON",.T.,.F.)
   Old_talk  = IIF(SET("TALK")="ON",.T.,.F.)
   Old_esc   = IIF(SET("ESCAPE")="OFF",.T.,.F.)
   Old_delet = IIF(SET("DELETE")="ON",.T.,.F.)
   Old_exact = IIF(SET("EXACT")="ON",.T.,.F.)
   Old_delim = IIF(SET("DELIMITERS")="ON",.T.,.F.)
   Old_head  = IIF(SET("HEADING")="ON",.T.,.F.)
   SET NEAR OFF
   SET BELL OFF
   SET STAT OFF
   SET SCORE OFF
   SET TALK OFF
   SET ESCAPE ON
   SET DELETE OFF
   SET EXACT OFF
   SET DELIMITERS OFF
   ON KEY LABEL F1 ?? CHR(7)
   ON KEY LABEL F2 ?? CHR(7)
   ON KEY LABEL F3 ?? CHR(7)
   ON KEY LABEL F4 ?? CHR(7)
   ON KEY LABEL F5 ?? CHR(7)
   ON KEY LABEL F6 ?? CHR(7)
   ON KEY LABEL F7 ?? CHR(7)
   ON KEY LABEL F8 ?? CHR(7)
   ON KEY LABEL F9 ?? CHR(7)
   ON KEY LABEL F10 ?? CHR(7)
   ON ERROR
   *-- Load the binary cursor control module.
   *-- The cammands to CALL CURSOR WITH "ON" ("OFF") are found in
   *-- procedure PickScan.
   LOAD CURSOR
   *-- Load the binary keyboard buffer stuffer module.
   LOAD KEYBD
   USE Poporder in 1
   USE Pickname Order L_name In 2                && Indexed on UPPER(L_name)
   USE Pickpart Order Id In 3                    && Indexed on ID
   GOTO TOP
   IF BOF()
      *-- If there are no records, create a blank record!
      APPEND blank
   ENDIF
   set procedure to PopPick5
RETURN

PROCEDURE D_WinPop
   *-- Programmers must maintain their windows/popups here
   DEFINE WINDOW Windlast FROM 9,50 TO 20,73 DOUBLE COLOR N/W,N/Bg,R/W
   DEFINE WINDOW Windname FROM 9,35 TO 20,79 DOUBLE COLOR N/W,N/Bg,R/W
   DEFINE WINDOW Windpart FROM 8,40 TO 21,77 DOUBLE COLOR N/W,N/Bg,R/W

   *-- Define the popup for use in demonstrating POPUP..PROMPT..FIELD
   *-- Note: POPUPs defined with PROMPT FIELD take up a ==LOT== of memory,
   *--       especially when used with a LARGE dbf.
   *--       When used with a large dbf, it can take quite a while to build
   *--       the associated files!

   DEFINE POPUP Pop_name FROM 9,50 TO 20,73 Prompt FIELD L_name ;
   Message " Navigate, or enter search key. "
   ON SELECTION POPUP Pop_name DEACTIVATE POPUP
RETURN

PROCEDURE Reset_env
   ** Housekeeping
   IF Old_con
      SET CONFIRM ON
   ENDIF
   IF Old_near
      SET NEAR ON
   ENDIF
   IF Old_bell
      SET BELL ON
   ENDIF
   IF Old_talk
      SET TALK ON
   ENDIF
   IF Old_stat
      SET STATUS ON
   ENDIF
   IF Old_score
      SET SCOREBOARD ON
   ENDIF
   IF Old_esc
      SET ESCAPE OFF
   ENDIF
   IF Old_delet
      SET DELETE ON
   ENDIF
   IF Old_exact
      SET EXACT ON
   ENDIF
   IF Old_delim
      SET DELIMITERS ON
   ENDIF
   IF Old_head
      SET HEADING ON
   ENDIF
   ON KEY LABEL F1
   ON KEY LABEL F2
   ON KEY LABEL F3
   ON KEY LABEL F4
   ON KEY LABEL F5
   ON KEY LABEL F6
   ON KEY LABEL F7
   ON KEY LABEL F8
   ON KEY LABEL F9
   ON KEY LABEL F10
   RELEASE WINDOW Windlast
   RELEASE WINDOW Windname
   RELEASE WINDOW Windpart
   RELEASE POPUP Pop_name
   RELEASE MODULE CURSOR
   RELEASE MODULE KEYBD
   SET FORMAT TO
   CLOSE ALL
   ON ERROR
   CLEAR
RETURN


PROCEDURE POP_FMT
   *-- These lines were imported from POPNPICK.FMT
   *-- and the appropriate line were "commented" out.

********************************************************************************
*-- Name.......: POPNPICK.FMT
*-- Date.......: 3-07-90
*-- Version....: dBASE IV, Format 1.1
*-- Notes......: Format files use "" as delimiters!
********************************************************************************

*-- Format file initialization code --------------------------------------------

*-- IF SET("TALK") = "ON"
*--    SET TALK OFF
*--    lc_talk = "ON"
*-- ELSE
*--    lc_talk = "OFF"
*-- ENDIF

*-- This form was created in EGA25 mode
*-- SET DISPLAY TO EGA25

*-- lc_status = SET("STATUS")
*-- SET STATUS was OFF when you went into the Forms Designer.
*-- IF lc_status = "ON"
*--    SET STATUS OFF
*-- ENDIF


*-- @ SAY GETS Processing. -----------------------------------------------------

 
*--  Format Page: 1
@ 1,19 TO 5,60 DOUBLE COLOR n/w 
*-- @ 2,20 SAY "          PopPick5 - Format File        " COLOR gr+/w 
@ 2,20 SAY "         PopPick5 - Procedure Demo      " COLOR gr+/w 
@ 3,19 SAY "Ķ" COLOR n/w
@ 4,20 SAY "          Press F3 for Pick List        " COLOR r/w 
@ 7,0 TO 15,79 DOUBLE COLOR r/w 
@ 8,1 SAY " " COLOR r/w 
@ 8,2 SAY "Last name: " COLOR n/w 
@ 8,13 GET Lname1 PICTURE "!XXXXXXXXXXXXXXXXXXX" ;
   VALID PICKFUNC(LNAME1) ;
   WHEN PickWhen(LNAME1) ;
   MESSAGE "Enter last name - F3 will use 1 field list to compare with POPUP..PROMPT..FIELD" 
@ 8,33 SAY "   <--Demonstrates 1 Field PopNPick List      " COLOR n/w 
@ 9,1 SAY "                                                                              " COLOR n/w 
@ 10,1 SAY " Last name: " COLOR n/w 
@ 10,13 GET Lname2 PICTURE "!XXXXXXXXXXXXXXXXXXX" ;
   VALID PICKFUNC(LNAME2) ;
   WHEN PickWhen(LNAME2) ;
   MESSAGE "Enter last name - F3 will use dBase IV's POPUP..PROMPT..FIELD" 
@ 10,33 SAY "   <- Demonstrates POPUP..PROMPT..FIELD List  " COLOR n/w 
@ 11,1 SAY "                                                                              " COLOR n/w 
@ 12,1 SAY " Last name: " COLOR n/w 
@ 12,13 GET Lname3 PICTURE "!XXXXXXXXXXXXXXXXXXX" ;
   VALID PICKFUNC(LNAME3) ;
   WHEN PickWhen(LNAME3) ;
   MESSAGE " Enter last name - F3 will display a 2 field lookup table." 
@ 12,33 SAY "   <--Demonstrates 2 Field PopNPick List      " COLOR n/w 
@ 13,1 SAY "                                                                              " COLOR n/w 
@ 14,1 SAY " Part ID..: " COLOR n/w 
@ 14,13 GET Id PICTURE "@! XXXXXX" ;
   VALID PICKFUNC(ID) ;
   WHEN PICKWHEN(ID) ;
   MESSAGE "  Enter part ID, Navigate, F3 for a 3 field lookup, <Esc> to abort." 
@ 14,19 SAY "                 <--Demonstrates 3 Field PopNPick List      " COLOR n/w 
*-- Calculated field: Del_flag - 
@ 18,36 SAY iif(DELETE(),"Deleted","       ") PICTURE "XXXXXXX" COLOR r+*/b 

*-- Format file exit code -----------------------------------------------------

*-- SET STATUS was OFF when you went into the Forms Designer.
*-- IF lc_status = "ON"  && Entered form with status on
*--   SET STATUS ON     && Turn STATUS "ON" on the way out
*-- ENDIF


*-- IF lc_talk="ON"
*--   SET TALK ON
*-- ENDIF

*-- RELEASE lc_talk,lc_fields,lc_status
*-- EOP: POPNPICK.FMT

RETURN
