*-- Name    : POPPICK5
*-- Author  : Richard H. Price
*--           Copyright (c) 1989, 1990 by Richard H. Price
*--           All procedures may be used free of charge, HOWEVER,
*--           if anyone feels that they have gained any benefit of any
*--           kind, any and all donations ($10.00 ?) will be welcomed!
*--           For those kind individuals:  P.O. Box 337
*--                                        Arrey, NM 87930
*-- Purpose : Procedural Library for dBASE IV ver 1.0 pick lists 
*-- Modified: 03/03/90
*-- Contact : via Ashton-Tate BBS user: HAMMETT 
*--
*--           The latest version of Pop-N-Pick!
*--
*--           This edition contains "work arounds" to the problems associated
*--           with using FORMAT files in conjunction with the READ statement.
*--           When FORMAT files are used with a READ, if a UDF has been or is
*--           utilized within the FORMAT file, then any changes made to 
*--           dbf fields will not be "written" to the dbf unless the field
*--           is "hard-coded" REPLACED. This condition (from here on called
*--           the "read bug") will continue until an EDIT, BROWSE, CHANGE,
*--           APPEND, or APPEND BLANK is issued.  The "read bug" will also
*--           make its appearance known if FORMAT/READ code is used and an
*--           ON KEY procedure is called.  Interestingly enough, however, is
*--           that the condition can be temporarily reversed and all changed
*--           fields will be "saved" by the issuance of an ON KEY DO <proc>
*--           prior to moving the record pointer in the dbf being "READ".
*--           This version of Pop-N-Pick contains code which allows one to
*--           use a FORMAT file to access records and to emulate EDIT NOHELP
*--           without the "read bug" effecting data integrity - as was the
*--           case with prior versions. If a FORMAT file is used, then the 
*--           code will activate a dummy FORMAT file, issue an APPEND, and
*--           escape out without creating a new record, thereby "killing"
*--           any "read bugs" which may be present.
*--           
*--           It should be noted here for those unfamiliar with prior versions
*--           and for those who haven't read the accompanying document file 
*--           that there are many problems associated with using the code
*--           required for pick lists (at least the code I use <grin>) and
*--           FORMAT files used with CHANGE, EDIT, BROWSE, and APPEND <blank>.
*--           
*--           Enhancements include having the pick list default to the closest
*--           match based upon the currently held data in the field when 
*--           the pick list was called, AND giving the programmer the 
*--           option of forcing a pick list upon the user when entering 
*--           a field whose data is required to be from those found in a 
*--           pick list (see FUNCTION PickWhen).

                             *****************
                             **  DISCLAMER  **
                             *****************

*--     This software is provided "AS IS" and by using this software or 
*--     any portion thereof, you agree that Richard H. Price and/or 
*--     Price's Dairies, Inc. will not be held liable for any reason for 
*--     any cost you may incur, whether in the present or the future.

*******************************************************************************
**  The following procedures and function may be placed in your library file **
**  or main procedure file in order to provide pick list help from within    **
**  any of your calling programs.                                            **
**                       *******************                                 **
**  PROCEDURE PickEdit  -  Emulate EDIT NOMENU                               **
**  PROCEDURE Pick_Del  -  Used with PickEdit for record deletion/recall     **
**  PROCEDURE Pick_Fix  -  A "do nothing" procedure used to temporarily      **
**                         clear the "read bug" in order to "save" changes.  **
**  FUNCTION  PickWhen  -  UDF: Calls a pick list when field is entered.     **
**  FUNCTION  PickFunc  -  UDF: Data validation & pick list call if required.**
**  PROCEDURE PickKey   -  Used to activate pick list via a "hot key" press. **
**  PROCEDURE PickList  -  Main calling program for pick list management.    **
**  PROCEDURE PickScan  -  Processes user keystrokes while in a pick list.   **
**  PROCEDURE PickCase  -  Sets working environment for pick lists, and is   **
**                         called from both Procedures PickList and PickScan **
**                         and FUNCTION PickFunc (for data validation)       **
**                         NOTE: Requires programmer maintenance.            **
**  PROCEDURE PickAbrt  -  No valid CASE statement found in PickCase, flag   **
**                         the appropiate memvars and abort pick lists.                           **
**                       *******************                                 **
**  NOTE: It is up to the programmer to place the WINDOW and POPUP           **
**        definition statements wherever is applicable ==PRIOR== to          **
**        the execution of any pick list routines!                           **
*******************************************************************************
PROCEDURE PickEdit
   PARAMETERS FMT_NAME,Is_Fmt

   *-- FMT_NAME is the name of ==EITHER== a FORMAT file =OR= a procedure
   *-- where @..SAY..GETs are used.
   *-- Is_Fmt is used to "flag" whether or not a FORMAT file is used.
   *-- If a format file is used, Is_Fmt = .t., otherwise, Is_Fmt = .f.
   *-- Mimics EDIT NOMENU by using READ.
   *-- Determine validity of passed parameters (as best as one can).
   IF ""=FMT_NAME
      *-- There was no parameter passed for either a format file or a procedure.
      *-- Therefor, this procedure is useless to the calling program!
      DEFINE WINDOW NoFormat from 8,11 to 16,68 double
      ACTIVATE WINDOW NoFormat
      ?? chr(7)+chr(7)
      @ 0,3 say "PROCEDURE PickEdit is TERMINATING!"
      @ 1,7 say "************"
      @ 2,2 say "This procedure requires either a FORMAT file name"
      @ 3,9 say "=OR="
      @ 4,3 say "a PROCEDURE name where screen formatting occurs!"
      @ 5,7 say "************"
      @ 6,4 say "Press any key to continue...."
      i=INKEY(0)
      DEACTIVATE WINDOW NoFormat
      RELEASE WINDOW NoFormat
      RETURN
   ENDIF

   *-- If a format file =IS= being utilized, activate it.
   IF Is_Fmt
      SET FORMAT TO &FMT_NAME
   ENDIF

   *-- Repeat the "editing" procedures for as long as the user wishes.
   DO WHIL .t.

      *-- Procedure Pick_Del is used for DELETE/RECALL during a READ.
      *-- Del_Flag is the "flag" used to indicate the record's DELETE() status
      *-- in order to more adequately mimic EDIT
      Del_Flag=delete()
      ON KEY LABEL CTRL-U DO Pick_Del WITH Del_Flag

      *-- Establish a "hot key" used to call up a pick list.
      ON KEY LABEL F3 DO PickList WITH .f.  && "",.f.,.f.

      IF .not. Is_Fmt
         *-- A format file is not in use, but a formating procedure is used!
         *-- Therefore, DO the screen painting & memvar/field GETting.
         DO &FMT_NAME
      ENDIF

      READ

      *-- Turn off the hot key used to implement a hot key pick list.
      *-- (I prefer to have it "beep" when pressed again.)
      ON KEY LABEL F3 ?? CHR(7)

      *-- Deactivate the DELETE RECORD key toggle.
      ON KEY LABEL CTRL-U

      *-- Initialize the variable used for subsequent processing.
      *-- All subsequent processing is based upon the key press used to exit
      *-- the READ!
      LK=lastkey()

      IF Is_Fmt .AND. READKEY()>255
         *-- Is_Fmt indicates that a format file is in use AND readkey()>255
         *-- indicates that data has been "edited".
         *-- Because the "READ/FORMAT" bug causes dBASE IV to NOT SAVE the
         *-- edited data, a "work around" must be used!
         *-- I found that even though the bug disallows changes to be written,
         *-- subsequent calls to procedures which unleash the bug also will
         *-- cause the read buffers to flush themselves for posting. The
         *-- effect is that any changes made prior to the next bug "call"
         *-- will be written, but subsequent changes will not - the bug
         *-- is back in place, but the prior changes which were affected by
         *-- the bug are now no longer effective, but future changes will be
         *-- affected. For now, issue the "bug call" to write the changes,

         *-- BTW, if an ESC is used to exit the read, and the "read bug" is
         *-- active, things work out rather nicely <grin> because the changed
         *-- data will not be kept and that makes this routine behave even
         *-- more like FORMAT/EDIT.

         *-- NOTE: Code issued immediately prior to RETURN in this procedure
         *--       will "kill" the bug before control returns to your calling
         *--       application.

         *-- First off, flush the current keyboard buffer.
         CLEAR TYPE
         FlushType=inkey()
         DO WHILE FlushType<>0
            FlushType=inkey()
         ENDDO
         RELEASE FlushType

         *-- Assign a hot key which calls a dummy procedure.
         *-- I opted to use the "|" key only because "it was there" <grin>.
         ON KEY LABEL | DO Pick_Fix

         *-- Mimic the pressing of the hot key.
         CALL KEYBD WITH "|"


      ENDIF

      *-- Check to see if the user pressed the DELETE toggle (Ctrl-U).
      DO CASE
         CASE DEL_FLAG .and. .not. DELETE()
            *-- The user pressed CTRL-U to delete a non deleted record.
            DELETE
            LOOP
         CASE .not. DEL_FLAG .and. DELETE()
            *-- The user pressed CTRL-U to recall a deleted record.
            RECALL
            LOOP
      ENDCASE

      *-- Position record pointer base on used keypress.
      DO CASE
         CASE Lk = 23 .OR. Lk = 27
            *-- Ctrl-End or Ctrl-W or Escape was pressed - End session!
            EXIT
         CASE Lk = 5 .OR. lk = 18 .or. Lk = 19 .or. Lk = 127 .or. Lk = -400
            *-- UpArrow/PgUp/LeftArrow/Backspace/Backtab
            SKIP -1
         CASE Lk = 3 .or. Lk = 9 .OR. Lk=13 .OR. lk = 24
            *-- PgDn/Tab/Enter/DownArrow
            SKIP
         CASE Lk = 31
            *-- Ctrl-PgUp
            GOTO TOP
         CASE Lk = 30
            *-- Ctrl-PgDn
            GOTO BOTTOM
         OTHERWISE
            SKIP
      ENDCASE
      IF BOF()
         GOTO TOP
      ENDIF
      IF EOF()
         *-- If EOF(), prompt for a new record.
         GOTO BOTT
         @ 23,20 say chr(7)+" Create a new record (Y/N) ? "
         *-- The use of the INKEY() function allows for a user response
         *-- due to the current usage of a screen format file.
         *-- If one was to issue an @..GET followed by a read,
         *-- the format file would be activated.
         I=INKEY(0)
         IF UPPER(CHR(I))="Y"
            APPEND blank
            LOOP
         ENDIF
      ENDIF
   ENDDO
   IF Is_Fmt
      *-- A FORMAT file is in use!

      *-- First off, flush the current keyboard buffer.
      CLEAR TYPE
      FlushType=inkey()
      DO WHILE FlushType<>0
         FlushType=inkey()
      ENDDO
      RELEASE FlushType

      *-- First, save your place in the dbf.
      Go_To_Rec = RECNO()

      *-- In order to "kill" the FORMAT/READ bug, one must issue
      *-- an EDIT, CHANGE, BROWSE, APPEND, or APPEND BLANK.

      *-- I prefer an APPEND followed by an ESCAPE with no user intervention.

      *-- Stuff the keyboard buffer with "Esc" in order to
      *-- quickly exit the "APPEND" and have no records added.

      *-- NOTE: One =could= use EDIT, but if there were, and there were
      *-- no LOGICAL records to EDIT, the EDIT call would do no good
      *-- and the bug would "survive" the extermination attempt!

      *-- In order to do things as unobtrusively as possible, instead of
      *-- using the default (provided that there is one) format file,
      *-- I opted to use a small format file which includes TEXT ONLY in order
      *-- to clear the bug condition.
      *-- Not only is the screen display quicker, but there is no chance of
      *-- field data being added/changed.

      *-- Set the format file to use to "whatever".
      SET FORMAT TO Kill_Bug

      T_Esc = IIF(SET("ESCAPE")="ON",.t.,.f.)
      SET ESCAPE OFF
      *-- Load the keyboard buffer with the ESCape key press.
      CALL KEYBD WITH CHR(27)
      APPEND
      IF T_Esc
         SET ESCAPE ON
      ENDIF
      RELEASE T_Esc
      *-- Deactivate the format file setting.
      SET FORMAT TO

      *-- Return to the record found prior to the APPEND statement.
      *-- Check first to see if it was a valid record.
      IF RECCOUNT()>1
         GO Go_To_Rec
      ENDIF
      RELEASE Go_To_Rec

   ELSE
      *-- If this procedure uses a procedure for formating the screen
      *-- with @...SAYS and @...GETS, you probably will want to clear it.
      CLEAR
   ENDIF
RETURN

PROCEDURE Pick_Del
   PARAMETER Del_Flag
   *-- This is called via CTRL-U when editing with a READ.
   *-- The calling procedure is PickRead
   *-- The purpose is to DELETE or RECALL a record while in READ, becuase
   *-- dBase IV does not recognize CTRL-U from READS, even when used with
   *-- a format file.
   *-- Set the "flag" to it's opposite condition.
   DEL_FLAG = iif(del_flag,.f.,.t.)
   *-- Replicate the pressing of Ctrl-End in order to delete/recall the record
   *-- upon re-processing as found in procedure PickEdit
   CALL KEYBD WITH CHR(23)
RETURN

PROCEDURE Pick_Fix
   *-- A do-nothing procedure activated by ON KEY LABEL | DO Pick_Fix
   *-- The dummy procedure has now been executed and the "edited"
   *-- data has been written (saved).
   *-- Disable the hot key for future use.
   On KEY LABEL |
RETURN

FUNCTION PickWhen
   PARAMETERS M_datachk
   *--     M_Datachk : Contains the data to be verified/posted.

   *-- This function can be used as a universal data validation function
   *-- when initially entering the FIELD/MEMVAR. A Pick List can/will be
   *-- provided if an invalid entry is found.

   *-- A typical usage of PickWhen would be to provide a pick list upon entry
   *-- into a newly appended record with a blank field.

   *-- This function must be used in conjunction with character data and with
   *-- proper maintenance of PROCEDURE PickCase.

   *-- Initialize required memvars:

   *--     M_Valid:  Will be set within PROCEDURE PickList
   *--               Determines the VALID condition for either
   *--               the current data in M_Datachk or the user's
   *--               pick list choice. (Will return .f. if a
   *--               pick list was used, but terminated with Esc)
   M_Valid = .f.

   *--     M_IsFunc: Flag which indicates to subsequent procedures that
   *-                they were "called" from a UDF (PickWhen or PickFunc)
   M_IsFunc = .t.

   *if .not. m_valid                             && ENABLE THESE 3 LINES WHEN MODIFYING A FORMAT FILE!
   *   RETURN .t.                                && ENABLE THESE 3 LINES WHEN MODIFYING A FORMAT FILE!
   *endif                                        && ENABLE THESE 3 LINES WHEN MODIFYING A FORMAT FILE!

   M_Varread = VARREAD()
   M_Alias   = ALIAS()

   *-- Select the proper work area for data validation.
   DO PickCase with "ENVIRONMENT"

   *-- Check to see if the entry was correct, and validate!
   *-- M_Valid will be flagged .t. if correct entry was made.
   DO PickCase with "VALIDATE"

   IF M_Valid
      *-- The data passed validation!
      *-- Reset the current work area.
      SELECT (M_Alias)

      do PICKCASE with "RESET"
   ELSE
      *-- The entry failed, call up a pick list.
      do PickList with .t.

      IF .NOT. M_valid
         *-- Bells and whistles. The user ESCaped from a pick list!
         ?? CHR(7)+CHR(7)

         *-- Get rid of "Press SPACE to continue..."
         CALL KEYBD WITH " " + CHR(26)
      ELSE
         *-- Set M_Valid to .F., in order to correctly display and fill
         *-- the active GET upon execution of the CALL KEYBD WITH <expC>
         M_Valid = .F.

         *-- Stuff the keyboard buffer to answer the "Press space bar..."
         *-- portion of an invalid return, and then fill in the memvar/field
         *-- with the lookup choice.

         CALL KEYBD with " " + CHR(26) + CHR(25) + RTRIM(M_Datachk)
         *--           <space>  <Home>   <Ctrl-Y>      <Data>      
      ENDIF
   ENDIF
RETURN .t.

FUNCTION PickFunc
   PARAMETERS M_datachk
   *--     M_Datachk : Contains the data to be verified/posted.

   *-- This function can be used as a universal data validation function, when
   *-- used in conjunction with character data and with proper maintenance of
   *-- of PROCEDURE PickCase
   *-- Initialize required memvars:

   *--     M_Valid:  Will be set within PROCEDURE PickList
   *--               Determines the VALID condition for either
   *--               the current data in M_Datachk or the user's
   *--               pick list choice. (Will return .f. if a
   *--               pick list was used, but terminated with Esc)
   M_Valid = .f.

   *--     M_IsFunc: Flag which indicates to subsequent procedures that
   *-                they were "called" from a UDF (PickWhen or PickFunc)
   M_IsFunc = .t.

   * if .not. m_valid                            && ENABLE THESE 3 LINES WHEN MODIFYING A FORMAT FILE!
   *     RETURN .t.                              && ENABLE THESE 3 LINES WHEN MODIFYING A FORMAT FILE!
   * endif                                       && ENABLE THESE 3 LINES WHEN MODIFYING A FORMAT FILE!

   M_Varread = VARREAD()
   M_Alias   = ALIAS()

   *-- Select the proper work area for data validation.
   DO PickCase with "ENVIRONMENT"

   *-- Check to see if the entry was correct, and validate!
   *-- M_Valid will be flagged .t. if correct entry was made.
   DO PickCase with "VALIDATE"

   IF M_Valid
      *-- The data passed validation!
      *-- Reset the current work area.
      SELECT (M_Alias)
      do PICKCASE with "RESET"
   ELSE
      *-- The entry failed, call up a pick list.
      do PickList with .t.

      IF .NOT. M_valid
         *-- Bells and whistles. The user ESCaped from a pick list!
         ?? CHR(7)+CHR(7)
         *-- Get rid of "Press SPACE to continue..."
         CALL KEYBD WITH " " + CHR(26)
      ELSE
         *-- Set M_Valid to .F., in order to correctly display and fill
         *-- the active GET upon execution of the CALL KEYBD WITH <expC>
         M_Valid = .F.

         *-- Stuff the keyboard buffer to answer the "Press space bar..."
         *-- portion of an invalid return, and then fill in the memvar/field
         *-- with the lookup choice.

         CALL KEYBD with " " + CHR(26) + CHR(25) + RTRIM(M_Datachk) 
         *--           <space>  <Home>   <Ctrl-Y>      <Data>
      ENDIF
   ENDIF
RETURN M_Valid

PROCEDURE PickList
   PARAMETERS M_IsFunc
   *-- PickList is always called with 3 parameters
   *-- Parameter 1 =  M_datachk is the data to be checked and returned.
   *-- Parameter 2 =  M_IsFunc is a "flag" used to indicate if the pick list
   *--                was called from a UDF() <Function PickFunc>.
   *--                M_IsFunc = .t. when called from a UDF()
   *-- Parameter 3 =  M_Valid is the VALID status returned to and by
   *--                FUNCTION PickFunc.

   *-- Memvar M_Valid will be flagged TRUE upon exit from Picklist
   *-- upon user pressing Enter or Ctrl-W or Ctrl-End, --OR- upon a valid
   *-- seek when used in conjunction with a UDF() validation.

   *-- Reset the ON KEY statement to "ring" the bell instead of
   *-- trying to re-execute the code!
   ON KEY LABEL F3 ?? CHR(7)
   IF .not. M_IsFunc
      *-- If PickList was called via Function PickFunc, these have been done!
      *-- Initialize memvars.
      M_Valid = .f.
      M_DataChk = ""
      M_Varread=VARREAD()
      M_Alias=ALIAS()
      DO PickCase WITH "ENVIRONMENT"
   ENDIF
   STORE .t. to M_PickScan
   STORE 0 TO Wind_rows, R_border, Key_len, Bar_pos
   DO PickCase with "OPENLIST"
   IF M_PickScan
      *-- PickScan SHOULD be .f. if a POPUP..PROMPT pick list was used
      DO PickScan
   ENDIF
   *-- Reset the environment
   SELECT (M_alias)
   DO PickCase WITH "RESET"

   *-- Reset the "hot key" used to bring up the pick list.
   ON KEY LABEL F3 DO PickList WITH .f.

   IF M_Valid .AND. .NOT. M_IsFunc
      *-- M_Valid was flagged .t. ==AND== the pick list was
      *-- called from an ON KEY command without the aid of a UDF().

      *-- Post the data from picklist to the active GET!
      CALL KEYBD with CHR(26) + CHR(25) + RTRIM(M_Datachk)
      *--             <Home>   <Ctrl-Y>      <Data>
   ENDIF
RETURN

PROCEDURE PickScan
   *-- PickScan displays the pick list and processes the user's
   *-- navigational keypresses.

   *-- NOTE: Because dBase IV doesn't reliably release memory after
   *--       usage, I have chosen not to assign values to memvars
   *--       which may be used to identify the user's "navigational"
   *--       keypress sequences.  The programs check for numerical values
   *--       only, while utilizing comments to identify what the values
   *--       relate to.

   *-- Turn the cursor OFF. (This assumes you have LOADed CURSOR.BIN !)
   CALL CURSOR WITH "OFF"

   *-- First off, verify window limits.
   R_Border = IIF(R_Border < 0, 0 ,R_Border)
   @ 0,R_Border SAY " "
   DO WHILE ROW() = 0
      R_border = r_border+1
      ?? " "
   ENDDO

   Wind_Rows = IIF(Wind_Rows < 1, 0, Wind_Rows-1)
   @ Wind_Rows,0 SAY " "
   ? " "

   DO WHILE (Wind_rows-1) <> ROW()
      Wind_rows = ROW()+1
      ?
   ENDDO

   *-- Test for a valid Key Length
   Key_len = IIF(Key_len <= 0, R_border, Key_Len)

   *-- Set operating contraints.
   GOTO BOTT
   Bot_rec = RECNO()
   GOTO TOP
   Top_rec = RECNO()
   *-- Go_to_rec is used to position the record pointer after both
   *-- displays and exiting this procedure.
   Go_to_rec = Top_rec
   Found_Rec = Go_To_Rec

   *-- Determine limits.
   *-- Max_list will determine the maximum number of lines
   *-- which may display in the listing window.
   *-- Max_list will also be compared to the window size (Wind_rows)
   *-- to determine navigational processing.
   Max_list = 0
   SCAN WHILE Max_list < Wind_rows
      Max_list = Max_list+1
   ENDSCAN

   *-- Max_bar will be the maximum (bottom most) row indicator that the
   *-- list may use.  Its value will be 1 less than the maximum number of
   *-- items which can be viewed at once.
   Max_bar = Max_list-1

   *-- Data (column) headings, if used, are to be displayed in a seperate
   *-- window.
   *-- Due to the use of the LIST command, do not list the field name at
   *-- the top of any LISTING. This provides for quicker and cleaner displays.
   Old_Head = IIF(SET("HEAD")="ON",.T.,.F.)
   Old_Space = IIF(SET("SPACE")="OFF",.T.,.F.)
   SET HEAD OFF
   SET SPACE ON

   *-- Initialize screen positioning/painting variables.
   Bar_pos = 0                                   && <row> where highlited choice will appear.
   Bar_old = 0                                   && previous val of Bar_pos to minimize "blinking".

   *-- Goto to the top record and perform the initial listing.
   GOTO TOP
   Line_items = Max_List

   *-- Display the initial look list.
   @ 0,0 say ""
   DO PickCase with "PAINT"

   *-- Reset the record pointer to correspond with the first displayed item.
   GOTO TOP
   IF Max_list < Wind_rows
      *-- The top row will be blank. Need to move records to the top of
      *-- the display window for "browsing" purposes.*

      *-- NOTE: Another ANAMOLY ???
      *--    When a listing is performed and the combined length of all
      *--    the listed fields PLUS all spaces between fields PLUS the space
      *--    found at each end =EQUALS= the width of the window, the first
      *--    print (?) statement is IGNORED!.
      *--    Test for this condition by checking for COL()=0 (wraparound)
      *--    and issue a print (will be ignored) so that the adjusting print
      *--    statement will not be ignored!
      IF COL() = 0
         ?
      ENDIF
      @ wind_rows-1,0 SAY ""
      ?
   ELSE
      @ wind_rows-1,R_Border SAY CHR(25)
   ENDIF

   In_Key = 0                                    && Hold Value Of INKEY()
   I_len  = 0                                    && Length of the currently keyed,
                                                 && and "growing" search key.

   *-- Set the "need to list" flag
   Needlist = .F.

   *-- Initialize the memvars used for user inputed seek functions.
   M_SEEK_TMP = ""
   M_Seek = TRIM(M_DataChk)

   *-- The following code positions the pick list at the closest match
   *-- to the entry found as follows:
   *--          1) If the pick list was activated via a "hot key":
   *--
   *--              a) Position on the "best match" based upon
   *--                 the data held in memvar/field prior to operator
   *--                 editing.
   *--
   *--          2) If the pick list was activated by a UDF
   *--
   *--              a) Position on the "best match" based on
   *--                 the edited memvar/field.

   DO WHILE .T.
      SEEK M_Seek
      IF FOUND() .or. LEN(M_Seek)=1
         EXIT
      ENDIF
      M_Seek = LEFT(M_Seek,LEN(M_Seek)-1)
   ENDDO

   IF .NOT. FOUND()
      Old_Near = IIF(SET("NEAR")="OFF",.T.,.F.)
      SET NEAR ON
      M_Seek = M_DataChk
      SEEK M_Seek
      M_Seek=""
      IF Old_Near
         SET NEAR OFF
      ENDIF
      @ 0,0 say ""
      ?? CHR(7)+CHR(7)
   ENDIF

   Found_rec = IIF(EOF(), TOP_REC, RECNO())
   *-- Now, perform the screen displays and record positioning
   *-- as based upon which memvar was being edited when POPNPICK
   *-- was requested.
   *-- Move record pointer to postion prior to SEEKS
   GOTO Go_to_rec

   *-- Set the bar and record position to the top of the current display.
   Bar_pos=0

   *-- Check to see if the currently displayed top record matches
   *-- the requested seek, or if a display "re-paint" must take place.
   SCAN WHILE Bar_pos < Max_bar
      IF RECNO() = Found_rec
         EXIT
      ENDIF
      Bar_pos = Bar_pos+1
   ENDSCAN

   IF RECNO() <> Found_rec
      *-- A screen paint is in order.
      *-- This procedure wil always attempt to place the highlite bar
      *-- at the top of the display.
      Bar_pos = 0
      GOTO Found_rec
      SKIP Max_bar
      IF EOF()
         SKIP -Max_list
         SCAN WHILE RECNO() <> Found_rec
            Bar_pos = Bar_pos+1
         ENDSCAN
         SKIP -Bar_pos
      ELSE
         SKIP -Max_bar
      ENDIF
      @ wind_rows-1,r_border   SAY " "
      @ Max_list-Line_Items, 0 SAY ""

      *-- Do the associated paint routine.
      M_UpAr = IIF(RECNO()=TOP_REC,.F.,.T.)
      DO PickCase WITH "PAINT"

      *-- Set the record pointer and highlite the current record.
      @ 0,r_border say IIF(M_UpAr,CHR(24)," ")
      @ wind_rows-1,r_border say IIF(RECNO()<>BOT_REC,chr(25)," ")

      *-- Reset Line_items to maximum value.
      Line_items = Max_list
      Needlist = .F.
   ENDIF

   *-- Set the record pointer.
   Go_to_rec = Found_rec
   GOTO Go_to_rec

   *-- Reinitialize the memvars used for user inputed seek functions.
   *-- This way, the hilited bar will be set to the closest match,
   *-- but the operator may start a new search by the immediate spelling
   *-- of any key field. 
   M_SEEK_TMP = ""
   M_Seek = ""
   I_len = 0

   *-- This is the end of the initial positioning routine!

   *-- This loop is where the main pick list "action" takes place!
   DO WHILE .T.
      *-- If the bar position has been changed, fix the display.
      *-- Reset the last highlited bar, and highlite the new data.
      IF ISCOLOR()
         @ Bar_old,0 FILL TO Bar_old,R_border COLOR n/W
         @ Bar_pos,0 FILL TO Bar_pos,R_border COLOR W+/Bg
      ELSE
         @ Bar_old,0 FILL TO Bar_old,R_border COLOR W+/N
         @ Bar_pos,0 FILL TO Bar_pos,R_border COLOR N/W+
      ENDIF

      *-- Reset the old bar position memvar <Bar_old>.
      Bar_old = Bar_pos

      *-- Display the built up string of user inputed key presses when the
      *-- user has entered data and not moved via navigational keys.
      IF "" <> M_seek
         IF ISCOLOR()
            @ Bar_pos,1 fill to Bar_pos,I_len COLOR W+/B
         ELSE
            @ Bar_pos,1 fill to Bar_pos,I_len COLOR W+/N
         ENDIF
      ENDIF

      *-- Get the next user keystroke.
      @ Bar_pos,0 SAY ""
      In_Key = INKEY(0)

      *-- Determine what needs to be done based on user input.
      DO CASE
         CASE In_Key=27 .OR. In_Key=13 .OR. In_Key=23
            *-- Escape / Enter / Ctrl-End was pressed
            *-- Exit loop.
            IF In_Key=13 .or. In_Key=23
               *-- Set flag for automatic transfer to current GET.
               M_Valid = .T.
            ENDIF
            EXIT
         CASE In_Key<0 .AND. In_Key>-400
            *-- Filter out any possible invalid response.
            ?? CHR(7)
            LOOP
         CASE (In_Key>=32 .AND. In_Key<126) .OR. ((In_Key=19 ;
            .OR. In_Key=127 .OR. In_key = -400) .AND. I_len>0)
            *-- User has entered a non-navigational key.
            *-- 32 thru 126=printable characters / 19=LeftArrow / 127=Backspace
            *-- Presumably, user wants to perform a search based on the keypress.
            *-- Do procedure FINDPICK to search for a valid
            *-- I_len = Length of the currently keyed "growing" search key.
            I_len = LEN(M_seek)
            Test_key = CHR(In_Key)

            *-- SEEK the first matching record (if any) that is based on
            *-- user keypresses for a key field match.
            *-- This search algorythm will work regardless of upper/lower
            *-- case indexing and data entry.
            *-- Create new seek variables based upon the concatenation of
            *-- the current seek variable with both UPPER and LOWER values of
            *-- the last keypress.
            *-- If the last key press was a BACKARROW or BACKSPACE,
            *-- the new seek variables will be the same as the last valid seek
            *-- variable less the rightmost character.
            *-- If the length of the last seek variable is 1, reset to "" and return.
            *-- Test and flag for erasure of last keypress (LeftArrow/Backspace/Backtab)
            Del_Last = IIF(In_Key=19 .OR. In_Key=127 .OR. In_Key=-400,.T.,.F.)
            DO CASE
               CASE Del_Last .AND. I_len>1
                  *-- Last key pressed was BackArrow or BackSpace, and length of key>1
                  I_len = I_len-1
                  M_seek_hi = LEFT(M_seek,I_len)
                  M_seek_lo = LEFT(M_seek,I_len)
               CASE Del_last .AND. I_len<=1
                  *-- Last key pressed was BackArrow or BackSpace, and length of key<=1
                  I_len = 0
                  STORE "" to M_SEEK
                  LOOP
               CASE I_len < Key_len
                  I_len = I_len + 1
                  *-- Set the upper and lower concatenated search strings.
                  M_seek_hi = M_seek + UPPER(Test_key)
                  M_seek_lo = M_seek + LOWER(Test_key)
               OTHERWISE
                  *-- If you get here, don't waste time with subsequent routines.
                  *-- I_Len = Key_len
                  *-- Remember: Key_len = Maximum length of index key expression.
                  STORE M_seek TO M_seek_hi, M_seek_lo
                  ?? chr(7)
                  loop
            ENDCASE
            *-- Save the current record position.
            Go_to_rec = RECNO()
            *-- Find a match based on last keypress forced UPPER.
            M_seek = M_seek_hi
            SEEK M_seek
            IF .NOT. FOUND()
               *-- If last search was unsuccessful, perform a search based on
               *-- based on last keypress being converted to LOWER.
               M_seek = M_seek_lo
               SEEK M_seek
            ENDIF
            IF .NOT. FOUND()
               *-- No match was found based on user keypress!
               *-- Reset record pointer and memvars to values held upon entering
               *-- this routine, and return to PICKLIST
               GOTO Go_to_rec
               ?? CHR(7)
               I_len = I_len - 1
               M_seek = LEFT(M_seek,I_len)
               LOOP
            ENDIF

            *-- The SEEK has been succesful!
            *-- Store the last held value to the seek memvars.
            STORE M_seek TO M_seek_hi, M_seek_lo
            Found_rec = RECNO()

            *-- Now, perform the screen displays and record positioning
            *-- as based upon which memvar was being edited when POPNPICK
            *-- was requested.
            *-- Move record pointer to postion prior to SEEKS
            GOTO Go_to_rec

            *-- Set the bar and record position to the top of the current display.
            SKIP -Bar_pos
            Bar_pos=0

            *-- Check to see if the currently displayed top record matches
            *-- the requested seek, or if a display "re-paint" must take place.
            SCAN WHILE Bar_pos < Max_bar
               IF RECNO() = Found_rec
                  EXIT
               ENDIF
               Bar_pos = Bar_pos+1
            ENDSCAN

            IF RECNO() <> Found_rec
               *-- A screen paint is in order.
               *-- This procedure wil always attempt to place the highlite bar
               *-- at the top of the display.
               Bar_pos = 0
               GOTO Found_rec
               SKIP Max_bar
               IF EOF()
                  SKIP -Max_list
                  SCAN WHILE RECNO() <> Found_rec
                     Bar_pos = Bar_pos+1
                  ENDSCAN
                  SKIP -Bar_pos
               ELSE
                  SKIP -Max_bar
               ENDIF
               Needlist=.T.
            ENDIF

            Go_to_rec = Found_rec

         CASE Top_rec=Bot_rec
            *-- If there is only ONE (1) valid record for the picklist,
            *-- ring the bell and loop. No need in wasting time checking
            *-- for other situations!
            ?? CHR(7)
            LOOP

         CASE RECNO()=Top_rec .AND. ;
            (In_Key=26 .OR. In_Key=5 .OR. In_Key=18 .OR. In_Key=19 ;
            .OR. In_key=29 .OR. In_Key=31 .OR. In_key=127 .OR. In_key=-400)
            *-- Current record position is at the beginning, and
            *-- user has requested continued backward movement.
            *-- No further motion is available, so "ring" the bell and loop.
            ?? CHR(7)
            LOOP
         CASE RECNO()=Bot_rec .AND. (In_key=2 .OR. In_Key=3 .OR. In_Key=4 ;
            .OR. In_Key=9 .OR. In_Key=24 .OR. IN_KEY=30 )
            *-- Current record position is at the last record, and
            *-- user has requested continued forward movement.
            *-- No further motion is available, so "ring" the bell and loop.
            ?? CHR(7)
            LOOP
         CASE In_Key=5 .OR. In_Key=19 .OR. In_Key=127
            *-- Uparrow OR Backarrow OR Backspace key was pressed.
            *-- Any time a cursor positioning key (arrow/page/backspace/etc)
            *-- is pressed, reset the memvar used for "seek" operations (based
            *-- upon user non-navigational key presses).
            M_seek=""

            *-- Reposition display bar one line up,
            *-- and reposition record pointer.
            SKIP -1
            Bar_pos=Bar_pos-1
            IF Bar_pos>=0
               *-- If the bar position is valid (0 thru max_bar), no need to
               *-- repaint the screen. Loop thru to next entry.
               LOOP
            ENDIF

            *-- The new position of the bar would be invalid if allowed
            *-- allowed to continue from here (Bar_pos < 0).
            *-- Remember, if the pointer is at the 1st logical record in the
            *-- index or tag, and SKIP -1 is issued, the RECNO() will be the
            *-- same and BOF() = .t.
            *-- NOTE: This is not the case for the LAST logical record when issueing
            *-- the command SKIP ##. The RECNO() will then equal RECCOUNT()+1
            *-- instead of being equal to the last logical RECNO() and EOF() = .t.
            *-- Set the memvar used for record post list positioning
            *-- to the valid record number as based on user keypress.

            Go_to_rec = RECNO()
            *-- Set the "need to repaint the list" memvar "flag" to "true".
            *-- Because the listing will require new data at the top, the
            *-- quickest and easiest method will be to LIST the next MAX_LIST
            *-- number of records.

            Needlist=.T.
            *-- Set the bar highlite vertical positioning memvar to the uppermost
            *-- valid condition (0)
            *-- ie: @ Bar_pos,0 fill to Bar_pos, R_border color n/w
            Bar_pos=0

         CASE In_Key=24 .OR. In_Key=4 .or. In_Key= 9
            *-- Downarrow OR Rightarrow key was pressed.
            M_seek=""
            Bar_pos=Bar_pos+1
            SKIP

            *-- Check the validity of the updated BAR POSITION (Bar_pos)
            *-- If the new position is a valid position, loop to return for
            *-- more user input.
            IF Bar_pos<=Max_bar
               LOOP
            ENDIF

            *-- The help list will require an additional item added to the
            *-- bottom of the list.  Because these listings take place in a
            *-- WINDOW, the most efficient way to handle this is merely to
            *-- issue the command LIST OFF <field(s)> NEXT 1. This will
            *-- force all other items to shift their respective position up
            *-- one row, and will force the top item off the list.
            *-- The net effect is a SCROLL DOWN of the item list.
            Line_items = 1
            Bar_pos = Max_bar
            Go_to_rec = RECNO()
            NeedList = .T.

         CASE Max_list < Wind_rows .AND. (In_Key=3 .OR. In_Key=30)
            *-- PageDown or Ctrl-PageDown was pressed, and the
            *-- maximum number of eligible list items is less than
            *-- the number of rows assigned to the listing window.
            M_seek = ""

            *-- In this case, the entire lookup table will fit in
            *-- the assigned window. Scrolling of line items is the
            *-- only available option. Therefore, go to the last
            *-- record and the maximum bar position.
            GOTO BOTT
            Bar_pos = Max_bar

         CASE Max_list < Wind_rows .AND. (In_Key=18 .OR. In_Key=31)
            *-- PageUp or Ctrl-PageUP was pressed, and the
            *-- maximum number of eligible list items is less than
            *-- the number of rows assigned to the listing window.
            M_seek = ""

            *-- In this case, the entire lookup table will fit in
            *-- the assigned window. Scrolling of line items is the
            *-- only available option. Therefore, go to the first
            *-- record and set the bar position to ZERO (the top of.
            GOTO TOP
            Bar_pos = 0

         CASE In_Key=18
            *-- PageUp was pressed.
            M_seek=""

            *-- If the current bar position is NOT at the top of the display,
            *-- postion the bar at the top, re-position the record pointer,
            *-- and loop back for more user input.
            IF Bar_pos>0
               SKIP -Bar_pos
               Bar_pos=0
               LOOP
            ENDIF

            *-- The bar position was already at the top of the list, so skip
            *-- backwards the appropriate number of records to enable a listing
            *-- of the previous Max_list number of records.
            SKIP -Max_list

            *-- Set the post listing record positioning memvar to the record number
            *-- which will correspond to the post listing highlited record.
            Go_to_rec = RECNO()

            *-- Set the "need to repaint the list" memvar "flag" to "true".
            Needlist=.T.

         CASE In_Key=3
            *-- PageDown has been pressed.
            M_seek = ""
            IF Bar_pos<Max_bar
               SKIP Max_bar-Bar_pos
               Bar_pos=Max_bar
               LOOP
            ENDIF
            SKIP Max_list
            Go_to_rec = RECNO()
            IF EOF()
               SKIP -1
               Go_to_rec = RECNO()
            ENDIF
            Needlist=.T.
            SKIP -Max_bar

         CASE In_Key=31 .or. In_Key=26
            *-- Ctrl-PgUp or Home was pressed.
            M_seek = ""
            *-- User wants to go to the top of the list.
            *-- First, skip backwards to the top of the current display.
            SKIP -Bar_pos

            *-- Then, check to see if the record pointer is at the beginning
            *-- record (Top_rec).
            *-- Remember, don't waste time repainting (listing) the display
            *-- if the current display is valid.
            *-- Set the "flag" accordingly.
            Needlist = IIF(RECNO() = Top_rec,.F.,.T.)

            *-- Set the record pointer to the TOP of the file, and set display
            *-- memvars to their appropriate values.
            GOTO TOP
            Bar_pos = 0
            Go_to_rec = RECNO()

         CASE In_Key=30 .or. In_Key=2
            *-- Ctrl-PageDown or END was pressed.
            M_seek = ""

            *-- User wants to go to the bottom of the list.
            *-- First, skip forward to the bottom of the current display.
            SKIP (Max_bar-Bar_pos)

            *-- Then, check to see if the record pointer is at the last
            *-- record (Bot_rec).
            *-- Remember, don't waste time repainting (listing) the display
            *-- if the current display is valid.
            *-- Set the "flag" accordingly.
            Needlist = IIF(RECNO() = Bot_rec,.F.,.T.)

            *-- Set the record pointer to the bottom of the file, and set
            *-- display memvars to their appropriate values.
            GOTO BOTTOM
            Bar_pos = Max_bar

            *-- If a new listing is required, set the current record to the
            *-- itself LESS the maximum list size less 1.
            IF Needlist
               Go_to_rec = RECNO()
               SKIP -Max_bar
            ENDIF
      ENDCASE

      *-- If Needlist has been flagged .T., then perform the screen (window)
      *-- repainting procedure.
      IF Needlist
         *-- Paint over the previous highlite
         @ wind_rows-1,r_border say " "
         IF ISCOLOR()
            @ Bar_old,0 FILL TO Bar_old,R_border COLOR N/W
         ELSE
            @ Bar_old,0 FILL TO Bar_old,R_border COLOR W+/N
         ENDIF

         *-- Position cursor to the row where the listing will start.
         @ Max_list-Line_Items, 0 SAY ""

         *-- Do the associated paint routine.
         M_UpAr = IIF(RECNO()=TOP_REC,.f.,.t.)
         DO PickCase with "PAINT"

         *-- Set the record pointer and highlite the current record.
         @ 0,r_border say IIF(M_UpAr,CHR(24)," ")
         @ wind_rows-1,r_border say IIF(RECNO()<>BOT_REC,chr(25)," ")
         GOTO Go_to_rec

         *-- Reset the line_items to maximum value.
         Line_items = Max_list
         Needlist=.F.
      ENDIF
   ENDDO
   IF Old_Head
      SET HEAD ON
   ENDIF
   IF Old_Space
      SET SPACE OFF
   ENDIF

   *-- Turn the cursor ON. (This assumes you have LOADed CURSOR.bin !)
   CALL Cursor WITH "ON"

   *--  Return to PROCEDURE PICKLIST.
RETURN

PROCEDURE PickCase
   PARAMETERS M_process
   *-- PickCase "controls" how the pick list is implemented and which dbfs,
   *-- indexes, fields, windows, ==OR== POPUPS are to be used.
   *-- PROCEDURE PickCase is executed from PROCEDUREs PickList AND PickScan.
   *-- This procedure uses "nested" case statements.
   *-- The primary case determines which variable is being read (M_varread).
   *-- Each sub case will set operating environments specific
   *-- to the string held in M_process.
   DO CASE
      CASE M_varread = "LNAME1"
         DO CASE
            CASE M_process = "ENVIRONMENT"
               *-- If PickList was called via a UDF, store the data held
               *-- in the memvar/field, else, the data that the memvar/field
               *-- held prior to any changes (hot key press)
               *-- This enables the pick lists to default to the 
               *-- "closest match" when activated.
               M_DataChk = IIF(M_IsFunc,M_DataChk,LName1)

               *-- Set up the working environment for the pick list.
               *-- Select the lookup work area/alias
               *-- REMEMBER: Your dbf ==MUST== already be open!
               SELECT PICKNAME

               *-- Select the associated ndx/mdx
               SET ORDER TO TAG L_NAME

            CASE M_process = "VALIDATE"
               *-- Programmers may include any data validation routines
               *-- applicable to their UDF() within this CASE option.
               *-- If the data is validated, update control and data memvars:
               *--         M_Valid  = .t.        && UDF PickFunc() will return .t.
               M_Valid = SEEK(M_Datachk)

            CASE M_process = "OPENLIST"

               *-- Initiate the display memvars.

               *-- The following: Wind_Rows, R_Border, and Key_Len
               *-- do not require new values...they are all assigned the
               *-- value of zero (0) in procedure PickList.
               *-- However, your pick lists will execute quicker by
               *-- assigning the correct value under "OPENLIST"
               *-- There is =NO= error checking included for values 
               *-- which are higher than the window can accomodate, and your
               *-- program will "bomb" when too large a value is given.
               *-- Procedure PickScan will determine the correct setting
               *-- for any value of equal or lesser than the window can 
               *-- accomodate.
               *-- Wind_rows is the number of LINE ITEMS used for listings.
               Wind_rows = 10

               *-- R_border is the Rightmost column postion in the window.
               R_border  = 19

               *-- Key_len is the maximum INDEX KEY length.
               Key_len   = 20

               *-- Open the main pick list window.
               ACTIVATE WINDOW Windlast

            CASE M_process = "PAINT"
               *-- This is where you specify what fields are to be shown
               *-- in the pick list window.
               *-- The memvar Line_Items is set from PickScan (the calling
               *-- procedure) and will either be equal to Wind_Rows
               *-- or 1, dependant on listing logic.
               LIST OFF L_name NEXT Line_Items
            CASE M_process = "RESET"
               *-- Reset the environment prior to returning to application.
               *-- Deactivate pick list window.
               DEACTIVATE WINDOW Windlast

               IF M_Valid
                  *-- Use pick list choice!
                  *-- M_Datachk = Alias->Return_Field_Name
                  M_Datachk = PICKNAME->L_name

                  *-- Even though the CALL KEYBD WITH <expC> stuffs the buffer
                  *-- and appears on the screen display to save the data,
                  *-- this is not the case when using a format file
                  *-- or when the pick list is used as part of a UDF().

                  *                REMEMBER:
                  *              ------------
                  *    If a dbf field is being READ, REPLACE it.
                  *    If a memvar is being READ, STORE it!

                  REPLACE Lname1 with M_datachk
               ENDIF
            OTHERWISE
               *-- Safety net - programmers CASE statements should
               *-- trap and execute prior to this level.
               *-- If these procedures are called without a CASE/sub-CASE
               *-- associated with the memvar/field, set appropriate flags
               *-- and RETURN.
               DO PickAbrt
         ENDCASE
      CASE M_varread = "LNAME2"
         *-- This is an example of how to use a POPUP as the pick list.
         DO CASE
            CASE M_process = "ENVIRONMENT"
               M_DataChk = IIF(M_IsFunc,M_DataChk,LName2)
               SELECT PICKNAME
               SET ORDER TO TAG L_NAME
            CASE M_process = "VALIDATE"
               M_Valid = SEEK(M_datachk)
            CASE M_process = "OPENLIST"
               *-- "Flag" the memvar M_PickScan to false - procedure PickScan
               *-- is not required due to the use of a POPUP!
               *-- REMEMBER: M_PickScan was set to .t. at the beginning of
               *--           Procedure PickList.
               M_PickScan = .F.

               SEEK LEFT(M_DataChk,1)
               IF FOUND().and..not. EOF()
                  CALL KEYBD WITH M_DataChk
               ENDIF
               *-- When using a POPUP, be sure to include
               *-- the command prior to calling this pick list:
               *-- ON SELECTION POPUP <popup_name> DEACTIVATE POPUP
               ACTIVATE POPUP POP_NAME

               *-- Set M_data to the PROMPT() when using POPUP..PROMPT..FIELD
               *-- Prompt will either hold the user's choice or "" (null)
               M_datachk = PROMPT()

               *-- Set the "flag" M_Valid to .t. if a valid choice was made.
               *-- NOTE: If Esc was used to exit the POPUP, then PROMPT()=""
               M_Valid=IIF(""=M_Datachk,.f.,.t.)

            CASE M_process = "PAINT"
               *-- This case will not be called due to using
               *-- POPUP..PROMPT..FIELD, but I left it here for continuity.
            CASE M_process = "RESET"
               IF M_Valid
                  REPLACE Lname2 with M_datachk
               ENDIF
            OTHERWISE
               DO PickAbrt
         ENDCASE
      CASE M_varread = "LNAME3"
         DO CASE
            CASE M_process = "ENVIRONMENT"
               M_DataChk = IIF(M_IsFunc,M_DataChk,LName3)
               SELECT PICKNAME
               SET ORDER TO TAG L_NAME
            CASE M_process = "VALIDATE"
               M_Valid = SEEK(M_datachk)
            CASE M_process = "OPENLIST"
               Wind_rows = 10
               R_border  = 42
               Key_len   = 20
               ACTIVATE WINDOW Windname
            CASE M_process = "PAINT"
               LIST OFF L_name,F_name NEXT Line_items
            CASE M_process = "RESET"
               DEACTIVATE WINDOW Windname
               IF M_Valid
                  M_datachk = PICKNAME->L_name
                  REPLACE Lname3 with M_datachk
               ENDIF
            OTHERWISE
               DO PickAbrt
         ENDCASE
      CASE M_varread = "ID"
         DO CASE
            CASE M_process = "ENVIRONMENT"
               M_DataChk = IIF(M_IsFunc,M_DataChk,ID)
               SELECT PICKPART
               SET ORDER TO TAG ID
            CASE M_process = "VALIDATE"
               M_Valid = SEEK(M_Datachk)
            CASE M_process = "OPENLIST"
               Wind_rows = 12
               R_border  = 35
               Key_len   = 6
               ACTIVATE WINDOW Windpart
               M_PickScan = .t.
            CASE M_process = "PAINT"
               LIST OFF Id,Desc,On_hand NEXT Line_items
            CASE M_process = "RESET"
               DEACTIVATE WINDOW Windpart
               IF M_Valid
                  M_datachk = PICKPART->ID
                  REPLACE Id with M_datachk
               ENDIF
            OTHERWISE
               DO PickAbrt
         ENDCASE
      OTHERWISE
         *-- Safety net - programmers CASE statements should
         *-- trap and execute prior to this level.
         *-- If these procedures are called without a CASE/sub-CASE
         *-- associated with the memvar/field, set appropriate flags
         *-- and RETURN.
         DO PickAbrt
   ENDCASE
RETURN

PROCEDURE PickAbrt
   *-- Abort the pick list operations!.
   *-- WARNING: If a picklist window has been activated, and an error
   *-- condition occured (ie: you got here!), then your window will
   *-- remain open. Once you have programmed & debugged all of your
   *-- pick lists (PickCase), the only time this is executed will be
   *-- if a user presses a "hot key" in a non-supported field/memvar,
   *-- and then you won't have to worry about this warning!
   m_Valid    = .t.
   m_PickScan = .f.
   m_isFunc   = .t.
RETURN

